
/* Definitions from missing header files */

#ifndef Py_MISSING_H
#define Py_MISSING_H
#ifdef __cplusplus
extern "C" {
#endif

PyAPI_DATA(PyTypeObject) PyMethod_Type;
PyAPI_DATA(PyTypeObject) PyRange_Type;
PyAPI_DATA(PyTypeObject) PyTraceBack_Type;

#ifdef __cplusplus
}
#endif
#endif /* !Py_MISSING_H */
